#!/usr/bin/python
from __future__ import with_statement
from __future__ import absolute_import
import vot
import sys
import tensorflow as tf
from tracker import Tracker, Model
import numpy as np

def get_axis_aligned_bbox(region):
    try:
        region = np.array([region[0][0][0], region[0][0][1], region[0][1][0], region[0][1][1],
                           region[0][2][0], region[0][2][1], region[0][3][0], region[0][3][1]])
    except:
        region = np.array(region)
    cx = np.mean(region[0::2])
    cy = np.mean(region[1::2])
    x1 = min(region[0::2])
    x2 = max(region[0::2])
    y1 = min(region[1::2])
    y2 = max(region[1::2])
    A1 = np.linalg.norm(region[0:2] - region[2:4]) * np.linalg.norm(region[2:4] - region[4:6])
    A2 = (x2 - x1) * (y2 - y1)
    s = np.sqrt(A1 / A2)
    w = s * (x2 - x1) + 1
    h = s * (y2 - y1) + 1
    x = cx - w/2
    y = cy - h/2
    return x, y, w, h

handle = vot.VOT("polygon")
Polygon = handle.region()

imagefile = handle.frame()
if not imagefile:
    sys.exit(0)

config_proto = tf.ConfigProto()
config_proto.gpu_options.allow_growth = True
with tf.Graph().as_default(), tf.Session(config=config_proto) as sess:
    model = Model(sess)
    tracker = Tracker(model)
    init_box = get_axis_aligned_bbox(Polygon)
    tracker.initialize(imagefile, init_box)
    while True:
        imagefile = handle.frame()
        if not imagefile:
            break
        bbox = tracker.track(imagefile)
        region = vot.Rectangle(bbox[0], bbox[1], bbox[2], bbox[3])
        handle.report(region)